@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,16 +8,15 @@
 """
 import logging
 import os
+import sys
 from os import getenv
 from os.path import expanduser, isfile
 
 from .constants import BACKEND_URL
 
-try:
-    # python2
+if sys.version_info[0] < 3:
     from ConfigParser import RawConfigParser, Error as ConfigError
-except ImportError:
-    # python3
+else:
     from configparser import RawConfigParser, Error as ConfigError
 
 LOGGER = logging.getLogger(__name__)
@@ -28,31 +27,35 @@ HOME_FILE_PATH = expanduser("~/.sqreen.ini")
 CONFIG_DEFAULT_VALUE = {
     # The backend URL
     "URL": BACKEND_URL,
-
+    # Default token to use
+    "TOKEN": None,
+    # If the token is an org token, default app_name
+    "APP_NAME": None,
     # Proxy to use if needed
     "PROXY_URL": None,
-
     # Should the client check the rules signatures
     "RULES_SIGNATURE": True,
-
     # Custom rules to be used in development
     "RULES": None,
-
     # Log configuration
     "LOG_LEVEL": "CRITICAL",
     "LOG_LOCATION": None,
-
     # Should sqreen protection run while application is tested?
     "RUN_IN_TEST": False,
-
     # Should sqreen report callback performance on newrelic?
     "REPORT_PERF_NR": False,
-
     # Initial features usually given by the environment
     "INITIAL_FEATURES": "{}",
-
     # Custom IP headers
     "IP_HEADER": None,
+    # Strip http referrer
+    "STRIP_HTTP_REFERER": False,
+    # Enable PII scrubbing (password, ...)
+    "STRIP_SENSITIVE_DATA": True,
+    # Sensitive keys to strip
+    "STRIP_SENSITIVE_KEYS": "password, secret, passwd, authorization, api_key, apikey, access_token",
+    # Sensitive regexp (matches items like credit cards)
+    "STRIP_SENSITIVE_REGEX": r"^(?:\d[ -]*?){13,16}$"
 }
 
 
@@ -111,8 +114,12 @@ class Config(object):
         docstring
         """
 
-        file_path = self.config_path or self._file_path_from_env() or self._file_path_from_home() \
+        file_path = (
+            self.config_path
+            or self._file_path_from_env()
+            or self._file_path_from_home()
             or self._file_path_from_local()
+        )
 
         if not file_path:
             return {}
@@ -123,12 +130,12 @@ class Config(object):
             config.read(file_path)
 
             config_dict = {}
-            for option in config.options('sqreen'):
-                config_dict[option.upper()] = config.get('sqreen', option)
+            for option in config.options("sqreen"):
+                config_dict[option.upper()] = config.get("sqreen", option)
 
             return config_dict
         except ConfigError:
-            LOGGER.debug('Error parsing config file %s', file_path)
+            LOGGER.debug("Error parsing config file %s", file_path)
             return {}
 
     @staticmethod
@@ -138,8 +145,10 @@ class Config(object):
         """
         env_config = {}
         for env_var in os.environ:
-            if env_var.startswith('SQREEN_'):
-                env_config[env_var.replace('SQREEN_', '', 1)] = os.environ[env_var]
+            if env_var.startswith("SQREEN_"):
+                env_config[env_var.replace("SQREEN_", "", 1)] = os.environ[
+                    env_var
+                ]
 
         return env_config
 
