@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -21,7 +21,9 @@ class BatchDeliverer(SimpleDeliverer):
     """ Class responsible for batching events before sending them to backend
     """
 
-    def __init__(self, session, batch_size, max_staleness, randomize_staleness=True):
+    def __init__(
+        self, session, batch_size, max_staleness, randomize_staleness=True
+    ):
         super(BatchDeliverer, self).__init__(session)
         self.batch_size = batch_size
         self.original_max_staleness = max_staleness
@@ -62,13 +64,13 @@ class BatchDeliverer(SimpleDeliverer):
 
         return time_seen is None
 
-    def deliver_batch(self):
+    def deliver_batch(self, resilently=True):
         """ Send the current batch to the backend.
         Reset the first_seen values and randomize the max_staleness
         """
         LOGGER.debug("Deliver batch")
         batch = [self._format_event(event) for event in self.current_batch]
-        self.session.post_batch(batch)
+        self.session.post_batch(batch, send_resiliently=resilently)
 
         # Clean current state
         self.current_batch = []
@@ -93,23 +95,25 @@ class BatchDeliverer(SimpleDeliverer):
         """
         if isinstance(event, RemoteException):
             formatted = event.to_dict()
-            formatted['event_type'] = 'sqreen_exception'
+            formatted["event_type"] = "sqreen_exception"
         elif isinstance(event, Attack):
             formatted = event.to_dict()
-            formatted['event_type'] = 'attack'
+            formatted["event_type"] = "attack"
         elif isinstance(event, RequestRecord):
             formatted = event.to_dict()
-            formatted['event_type'] = 'request_record'
+            formatted["event_type"] = "request_record"
         else:
-            raise NotImplementedError('Unknown event type {}'.format(type(event)))
+            raise NotImplementedError(
+                "Unknown event type {}".format(type(event))
+            )
 
         return formatted
 
-    def drain(self):
+    def drain(self, resiliently):
         """ Drain current batch if not empty
         """
         if self._not_empty_batch():
-            self.deliver_batch()
+            self.deliver_batch(resilently=resiliently)
 
     def tick(self):
         """ Periodic check to send batch when needed
@@ -124,13 +128,13 @@ class BatchDeliverer(SimpleDeliverer):
         if not isinstance(event, RequestRecord):
             return [cls._event_key(event)]
         result = []
-        observed_dict = event.payload.get('observed', {})
-        for attack_dict in observed_dict.get('attacks', []):
-            result.append('att-{}'.format(attack_dict['rule_name']))
-        for exc_dict in observed_dict.get('sqreen_exceptions', []):
-            result.append('rex-{}'.format(exc_dict['klass']))
-        for sdk_entry in observed_dict.get('sdk', []):
-            result.append('sdk-{}'.format(sdk_entry[0]))
+        observed_dict = event.payload.get("observed", {})
+        for attack_dict in observed_dict.get("attacks", []):
+            result.append("att-{}".format(attack_dict["rule_name"]))
+        for exc_dict in observed_dict.get("sqreen_exceptions", []):
+            result.append("rex-{}".format(exc_dict["klass"]))
+        for sdk_entry in observed_dict.get("sdk", []):
+            result.append("sdk-{}".format(sdk_entry[0]))
         return result
 
     @classmethod
@@ -142,7 +146,9 @@ class BatchDeliverer(SimpleDeliverer):
         elif isinstance(event, Attack):
             return "att-{}".format(event.rule_name)
         else:
-            raise NotImplementedError('Unknown event type {}'.format(type(event)))
+            raise NotImplementedError(
+                "Unknown event type {}".format(type(event))
+            )
 
     ###
     # Various checkers used for checking if current batch should be sent
@@ -163,5 +169,7 @@ class BatchDeliverer(SimpleDeliverer):
     def _stale(self):
         """ Check if one event has been seen more than max_staleness in the past
         """
-        min_seen = min(self.first_seen.values())
-        return (min_seen + self.max_staleness) < time()
+        if self.first_seen:
+            min_seen = min(self.first_seen.values())
+            return (min_seen + self.max_staleness) < time()
+        return False
