@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -36,7 +36,7 @@ class SimpleDeliverer(object):
             err_msg = "Unknown event type {}".format(type(event))
             raise NotImplementedError(err_msg)
 
-    def drain(self):
+    def drain(self, resiliently):
         """ Since everything is posted at once nothing needs to be done here
         """
         pass
