@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -15,12 +15,16 @@ def get_real_user_ip(remote_addr, *ips):
     """
     private_ip = None
     for list_ips in ips:
-        for ip in list_ips.split(','):
+        for ip in list_ips.split(","):
             ip = ip.strip()
             if not ip:
                 # Dummy IP address, skip it.
                 continue
             try:
+                # Azure append the port to IPv4 in XFF headers. We need to strip it
+                if '.' in ip and ':' in ip:
+                    ip = ip.split(':')[0]
+
                 ip = ip_address(ip)
             except ValueError:
                 continue
