@@ -1,15 +1,20 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
 #
 """ Collect and aggregate metrics
 """
+import math
+from collections import Counter, defaultdict, namedtuple
 from datetime import timedelta
+from logging import getLogger
 
-from .actions import ActionName
 from .exceptions import SqreenException
+from .utils import truncate_time
+
+LOGGER = getLogger(__name__)
 
 
 class BaseAggregator(object):
@@ -17,34 +22,47 @@ class BaseAggregator(object):
 
     It contains the methods to update a period with new data and finalize the
     data just before uploading.
-    Aggregators doesn't store any state, it is passed by the MetricsStore.
+    Aggregators don't store any state, it creates, updates, and finizalises
+    an accumulator that is passed by the MetricsStore.
     """
 
     name = "Base"
 
-    def update(self, key, value, data=None):
-        """ Update the whole data for the period. For updating only the key,
-        override update_key instead.
-        """
-        if data is None:
-            data = {}
+    @staticmethod
+    def default_value_factory():
+        return None
 
-        data[key] = self.update_key(value, data.get(key))
-        return data
+    @classmethod
+    def initialize(cls, **options):
+        """ Initialize the accumulator.
+        """
+        return defaultdict(cls.default_value_factory)
 
     @staticmethod
-    def update_key(value, data=None):
+    def is_empty(accumulator):
+        return len(accumulator) == 0
+
+    @classmethod
+    def update(cls, key, value, accumulator):
+        """ Update the accumulator for the period. For updating only the key,
+        override update_key instead.
+        """
+        accumulator[key] = cls.update_key(value, accumulator[key])
+        return accumulator
+
+    @staticmethod
+    def update_key(new_value, cur_value):
         """ Update the data for a key in the period. For updating the whole data,
         override update instead.
         """
         raise NotImplementedError("")
 
-    @staticmethod
-    def finalize(data):
+    @classmethod
+    def finalize(cls, accumulator):
         """ Finalize the period, by default return the data. Could be override
         to compute a ration or sum some elements.
         """
-        return data
+        return accumulator
 
 
 class CollectAggregator(BaseAggregator):
@@ -53,26 +71,44 @@ class CollectAggregator(BaseAggregator):
     """
 
     name = "Collect"
+    default_value_factory = list
 
     @staticmethod
-    def update_key(value, data=None):
-        if data is None:
-            data = []
-
-        data.append(value)
-        return data
+    def update_key(new_value, cur_value):
+        cur_value.append(new_value)
+        return cur_value
 
 
 class SumAggregator(BaseAggregator):
 
     name = "Sum"
+    default_value_factory = int
 
     @staticmethod
-    def update_key(value, data=None):
-        if data is None:
-            data = 0
+    def update_key(new_value, cur_value):
+        return cur_value + new_value
 
-        return data + value
+
+class ExtensionPerformanceMetricAggregator(BaseAggregator):
+
+    name = "ExtensionPerformance"
+
+    @classmethod
+    def initialize(cls, **options):
+        return {'v': Counter()}
+
+    @staticmethod
+    def is_empty(accumulator):
+        return len(accumulator["v"]) == 0
+
+    @classmethod
+    def update(cls, key, value, accumulator):
+        """ Ignore key
+        """
+        accumulator['u'] = value['u']
+        accumulator['b'] = value['b']
+        accumulator['v'].update(value['v'])
+        return accumulator
 
 
 class AverageAggregator(BaseAggregator):
@@ -80,24 +116,77 @@ class AverageAggregator(BaseAggregator):
     name = "Average"
 
     @staticmethod
-    def update_key(value, data=None):
-        if data is None:
-            data = {'sum': 0, 'count': 0}
-
-        data['sum'] += value
-        data['count'] += 1
-        return data
+    def default_value_factory():
+        return {"sum": 0, "count": 0}
 
     @staticmethod
-    def finalize(data):
+    def update_key(new_value, cur_value):
+        cur_value["sum"] += new_value
+        cur_value["count"] += 1
+        return cur_value
+
+    @classmethod
+    def finalize(cls, accumulator):
         final_data = {}
 
-        for key, value in data.items():
-            final_data[key] = value['sum'] / float(value['count'])
+        for key, value in accumulator.items():
+            final_data[key] = value["sum"] / float(value["count"])
 
         return final_data
 
 
+BinningAccumulator = namedtuple("BinningAccumulator", [
+    "unit", "base", "inv_log_base", "sub_parcel", "values"
+])
+
+
+class BinningAggregator(BaseAggregator):
+    """Sample numbers into logarithmic bins."""
+
+    name = "Binning"
+
+    @classmethod
+    def initialize(cls, **options):
+        unit = options.get("unit")
+        base = options.get("base")
+        if unit is None or not unit > 0.0:
+            raise InvalidOptionAggregator("unit must be a positive value")
+        if base is None or not base > 1.0:
+            raise InvalidOptionAggregator("base must be greater than 1.0")
+        log_base = math.log(base)
+        log_unit = math.log(unit)
+        return BinningAccumulator(unit, base, 1 / log_base, log_unit / log_base, Counter())
+
+    @staticmethod
+    def is_empty(accumulator):
+        return len(accumulator.values) == 0
+
+    @classmethod
+    def update(cls, key, value, accumulator):
+        # key is ignored
+        if value < accumulator.unit:
+            value_bin = 1
+        else:
+            value_bin = 2 + math.floor(math.log(value) * accumulator.inv_log_base
+                                       - accumulator.sub_parcel)
+        accumulator.values[value_bin] += 1
+        accumulator.values["max"] = max(accumulator.values["max"], value)
+        return accumulator
+
+    @classmethod
+    def finalize(cls, accumulator):
+        return {
+            "u": accumulator.unit,
+            "b": accumulator.base,
+            "v": dict(accumulator.values.items()),
+        }
+
+
+class InvalidOptionAggregator(SqreenException):
+    """ Exception raised when an invalid option was passed to an aggregator.
+    """
+
+
 class UnknownAggregator(SqreenException):
     """ Exception raised when trying to register a metric with an unknown
     aggregation kind
@@ -121,7 +210,10 @@ class UnregisteredMetric(SqreenException):
     """
 
 
-PRODUCTION_AGGREGATORS = [SumAggregator(), AverageAggregator()]
+PRODUCTION_AGGREGATORS = [SumAggregator(),
+                          AverageAggregator(),
+                          ExtensionPerformanceMetricAggregator(),
+                          BinningAggregator()]
 
 
 class MetricsStore(object):
@@ -144,36 +236,68 @@ class MetricsStore(object):
         self.metrics = {}
         self.aggregators = {}
 
-    def register_metric(self, name, kind, period):
+    def register_metric(self, name, kind, period, **options):
         """ Register a new metric
         """
+        LOGGER.debug("Register metric '%s'", name)
+
         if name in self.metrics:
             existing_metric = self.metrics[name]
-            saved_metric_kind = existing_metric['kind']
+            saved_metric_kind = existing_metric["kind"]
             if saved_metric_kind != kind:
-                err_msg = "Metric '{}' has already been registered with kind {}"
-                raise AlreadyRegisteredMetric(err_msg.format(name, saved_metric_kind))
+                err_msg = (
+                    "Metric '{}' has already been registered with kind {}"
+                )
+                raise AlreadyRegisteredMetric(
+                    err_msg.format(name, saved_metric_kind)
+                )
 
             # Update the period
-            existing_metric['period'] = period
+            existing_metric["period"] = period
+            # Reset the metric if the options have changed
+            if options != existing_metric["options"]:
+                existing_metric["options"] = options
+                self.metrics[name] = self.reset_metric(existing_metric)
         else:
             if kind not in self.aggregators:
-                raise UnknownAggregator("Unknown aggregation kind: {}".format(kind))
-            self.metrics[name] = self._new_metric(kind, period)
+                raise UnknownAggregator(
+                    "Unknown aggregation kind: {}".format(kind)
+                )
+            metric = self._new_metric(kind, period, options)
+            self.metrics[name] = self.reset_metric(metric)
+
+    def unregister_metric(self, name):
+        """ Unregister a metric
+        """
+        LOGGER.debug("Unregister metric %r", name)
+        self.metrics.pop(name, None)
+
+    def ensure_ext_perf_metric(self, name, period):
+        if name in self.metrics:
+            return
+
+        self.register_metric(name, 'ExtensionPerformance', period)
 
     @staticmethod
-    def _new_metric(kind, period):
+    def _new_metric(kind, period, options):
         """ Return a dict for an empty metric period
         """
-        return {"kind": kind, "period": period,
-                "observation": None, "start": None}
+        return {
+            "kind": kind,
+            "period": period,
+            "observation": None,
+            "start": None,
+            "options": options,
+        }
 
     def register_aggregator(self, name, aggregator_function):
         """ Register a new aggregator under the name passed in input
         """
         if name in self.aggregators:
             msg = "Aggregator '{}' has already been registered to: {}"
-            raise AlreadyRegisteredAggregator(msg.format(name, self.aggregators[name]))
+            raise AlreadyRegisteredAggregator(
+                msg.format(name, self.aggregators[name])
+            )
         self.aggregators[name] = aggregator_function
 
     def register_production_aggregators(self):
@@ -186,13 +310,9 @@ class MetricsStore(object):
         """ Register production default metrics
         """
 
-        # Call count metric
         self.register_metric("sqreen_call_counts", "Sum", 60)
-        # Whitelist metric
         self.register_metric("whitelisted", "Sum", 60)
-        # Action metrics.
-        self.register_metric('action|' + ActionName.IP_BLACKLIST, 'Sum', 60)
-        self.register_metric('action|' + ActionName.IP_REDIRECT, 'Sum', 60)
+        self.register_metric("request_overtime", "Sum", 60)
 
     def update(self, name, at, key, value):
         """ Logic behind metric updating.
@@ -207,14 +327,11 @@ class MetricsStore(object):
         except KeyError:
             raise UnregisteredMetric("Unknown metric {}".format(name))
 
+        # Check if the metric should be published or not
+        metric = self.check_metric_period_over(metric, name, at, False)
         # Update start time if not set already (registered but never updated)
         if metric["start"] is None:
-            metric["start"] = at
-
-        # Check if the metric should be published or not
-        else:
-            metric = self.check_metric_period_over(metric, name, at, False)
-
+            metric["start"] = truncate_time(at, round_to=metric["period"])
         self._update_metric(metric, key, value)
 
     def _update_metric(self, metric, key, value):
@@ -231,10 +348,12 @@ class MetricsStore(object):
         finalize it and returns the new one.
         If force_finalize if False, check that the metric period is over first.
         """
-        if not metric["start"]:
+        if metric["start"] is None:
             return metric
 
-        period_over = metric["start"] + timedelta(seconds=metric["period"]) < at
+        period_over = (
+            metric["start"] + timedelta(seconds=metric["period"]) < at
+        )
         if force_finalize or period_over:
             return self.finalize_period(metric, name, at)
 
@@ -246,7 +365,9 @@ class MetricsStore(object):
         If force_finalize if False, check that the metric period is over first.
         """
         for metric_name, metric in self.metrics.items():
-            self.check_metric_period_over(metric, metric_name, at, force_finalize)
+            self.check_metric_period_over(
+                metric, metric_name, at, force_finalize
+            )
 
     def finalize_period(self, metric, name, at):
         """ Finalize a metric period. For each registered metric, call the
@@ -255,25 +376,35 @@ class MetricsStore(object):
 
         Called if either the period time was crossed or can forced on logout.
         """
+        aggregator = self.aggregators[metric["kind"]]
         # If no data has been gathered
-        if metric["observation"] is not None:
+        if not aggregator.is_empty(metric["observation"]):
             # Retrieve the current period
             # Call the finalize method on the aggregator
-            aggregator = self.aggregators[metric["kind"]]
 
             finished = {}
-            finished["observation"] = aggregator.finalize(metric["observation"])
-            finished["finish"] = at
+            finished["observation"] = aggregator.finalize(
+                metric["observation"]
+            )
+            usual_finish = metric["start"] + timedelta(seconds=metric["period"])
+            finished["finish"] = at if at < usual_finish else usual_finish
             finished["name"] = name
             finished["start"] = metric["start"]
             self.store.append(finished)
 
         # Reset the period
-        metric = self._new_metric(metric["kind"], metric['period'])
-        metric["start"] = at
+        metric = self.reset_metric(metric, at)
         self.metrics[name] = metric
         return metric
 
+    def reset_metric(self, metric, at=None):
+        if at is not None:
+            at = truncate_time(at, round_to=metric["period"])
+        aggregator = self.aggregators[metric["kind"]]
+        metric["observation"] = aggregator.initialize(**metric["options"])
+        metric["start"] = at
+        return metric
+
     def get_data_to_publish(self, at, force_finalize=True):
         """ Return the list of finished periods, reset the list of
         finished periods after.
@@ -291,7 +422,13 @@ class MetricsStore(object):
     def list_metrics(self):
         """ Return the list of registered metrics
         """
-        return self.metrics.keys()
+        # The list must be a copy so that we can still register / unregister metrics.
+        return list(self.metrics.keys())
+
+    def is_registered(self, name):
+        """ Return True if the metric was already registered.
+        """
+        return name in self.metrics
 
     def get_metric_kind(self, name):
         """ Return the kind of a given metric
