@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,24 +8,18 @@
 """
 from logging import getLogger
 
-from ..frameworks.flask_framework import FlaskRequest
+from ..frameworks.flask_framework import FlaskRequest, FlaskResponse
 from .record_request_context import RecordRequestContext
 
 LOGGER = getLogger(__name__)
 
 
 class RecordRequestContextFlask(RecordRequestContext):
-
-    def pre(self, original, *args, **kwargs):
+    def pre(self, instance, args, kwargs, **options):
         from flask import request
+
         self._store_request(FlaskRequest(request))
 
-    def post(self, *args, **kwargs):
+    def post(self, instance, args, kwargs, result=None, **options):
+        self.storage.store_response(FlaskResponse(result))
         self._clear_request()
-
-    @staticmethod
-    def failing(*args, **kwargs):
-        """ Post is always called in a Flask Middleware, don't clean the
-        request right now as it may be needed in a post callback
-        """
-        pass
