@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,19 +8,16 @@
 """
 from logging import getLogger
 
-from ..frameworks.pyramid_framework import PyramidRequest
+from ..frameworks.pyramid_framework import PyramidRequest, PyramidResponse
 from .record_request_context import RecordRequestContext
 
 LOGGER = getLogger(__name__)
 
 
 class RecordRequestContextPyramid(RecordRequestContext):
+    def pre(self, instance, args, kwargs, **options):
+        self._store_request(PyramidRequest(args[0]))
 
-    def pre(self, original, request):
-        self._store_request(PyramidRequest(request))
-
-    def post(self, *args, **kwargs):
-        self._clear_request()
-
-    def failing(self, *args, **kwargs):
+    def post(self, instance, args, kwargs, result=None, **options):
+        self.storage.store_response(PyramidResponse(result))
         self._clear_request()
