@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -8,18 +8,22 @@
 """
 
 import logging
+import sys
 from datetime import datetime
 from time import time
 
+from sqreen.performance_metrics import PerformanceMetricsSettings
+
 from .actions import ACTION_STORE
 from .deliverer import get_deliverer
-from .list_filters import IPNetworkListFilter, RequestPrefixListFilter
+from .ip_radix import Radix
+from .list_filters import RequestPrefixListFilter
+from .metrics import UnregisteredMetric
 
-try:
-    # Python2
-    from queue import Empty, Queue, Full
-except ImportError:
+if sys.version_info[0] < 3:
     from Queue import Empty, Queue, Full
+else:
+    from queue import Empty, Queue, Full
 
 MAX_QUEUE_LENGTH = 100
 MAX_OBS_QUEUE_LENGTH = 1000
@@ -92,16 +96,19 @@ class RunnerSettings(object):
     """
 
     def __init__(self):
-        self.ips_whitelist = IPNetworkListFilter()
+        self.ips_whitelist = Radix(None)
         self.paths_whitelist = RequestPrefixListFilter()
+        self.debug_flag = None
 
     def set_ips_whitelist(self, addresses):
         """ Replace the current list of whitelisted IP networks
         """
-        self.ips_whitelist.reset(addresses)
+        self.ips_whitelist.clear()
+        for address in addresses:
+            self.ips_whitelist.insert(address, '/' in address)
 
     def ips_whitelist_match(self, address):
-        """ Return the first matching IP network, or None if no matchcould be
+        """ Return the first matching IP network, or None if no match could be
         found.
         """
         if address:
@@ -116,46 +123,57 @@ class RunnerSettings(object):
         """ Return the first matching request path prefix, or None if no match
         could be found.
         """
-        return self.paths_whitelist.match(path)
+        if path is not None:
+            return self.paths_whitelist.match(path)
 
     def whitelist_match(self, request):
         """ Return the first matching IP network or request path prefix, or
         None if no match could be found.
         """
-        ip = self.ips_whitelist_match(request.client_ip)
+        ip = self.ips_whitelist_match(request.raw_client_ip)
         if ip is not None:
             return ip
         prefix = self.paths_whitelist_match(request.path)
         if prefix is not None:
             return prefix
 
+    def set_debug_flag(self, flag):
+        """ Set to True if the framework is in debug mode.
+        """
+        self.debug_flag = flag
+
+    def get_debug_flag(self):
+        """ Get the current debug flag.
+        """
+        return self.debug_flag
+
 
 def process_initial_commands(initial_payload, runner):
     """ Process the initial non-standard login payload
     """
-    commands = initial_payload.get('commands', [])
+    commands = initial_payload.get("commands", [])
+    rulespack_id = initial_payload.get("pack_id", None)
+    rules = initial_payload.get("rules", None)
 
     # Preprocess commands first
     for command in commands:
-        if command["name"] == "instrumentation_enable":
-            rulespack_id = initial_payload.get('pack_id')
-            rules = initial_payload.get('rules')
-
-            if not rulespack_id or not rules:
-                msg = "Invalid login payload, pack_id %s, rules %s"
-                LOGGER.warning(msg, rulespack_id, rules)
-            else:
-                command['params'] = (rulespack_id, rules)
+        if command["name"] == "instrumentation_enable" and not command["params"]:
+            command["params"] = (rulespack_id, rules)
 
     # Process commands normally
     if commands:
         runner.process_commands(commands)
 
-    actions = initial_payload.get('actions', [])
-    ACTION_STORE.reload_from_dicts(actions)
+    actions = initial_payload.get("actions")
+    ACTION_STORE.reload_from_dicts(actions or [])
 
 
-class Runner(object):
+class BaseRunner(object):
+    """ Base runner class
+    """
+
+
+class Runner(BaseRunner):
     """ Main runner class
 
     Its job is to be the orchestrator and receiver for application communication
@@ -166,10 +184,23 @@ class Runner(object):
     # Heartbeat delay is 5 minutes by default
     HEARTBEAT_DELAY = 300
 
-    def __init__(self, queue, observation_queue, session, deliverer,
-                 remote_command, runtime_infos, instrumentation, metrics_store,
-                 settings, initial_features=None):
-        self.logger = logging.getLogger('{}.{}'.format(self.__module__, self.__class__.__name__))
+    def __init__(
+        self,
+        queue,
+        observation_queue,
+        session,
+        deliverer,
+        remote_command,
+        runtime_infos,
+        instrumentation,
+        metrics_store,
+        settings,
+        initial_features=None,
+        performance_metrics=True,
+    ):
+        self.logger = logging.getLogger(
+            "{}.{}".format(self.__module__, self.__class__.__name__)
+        )
         self.queue = queue
         self.observation_queue = observation_queue
         self.deliverer = deliverer
@@ -179,6 +210,8 @@ class Runner(object):
         self.metrics_store = metrics_store
         self.settings = settings
         self.stop = False
+        # Used to globally disable performance metrics in the agent from the daemon
+        self.performance_metrics = performance_metrics
 
         # Save the time runner started for checking warmup period termination
         self.started = time()
@@ -187,15 +220,21 @@ class Runner(object):
             initial_features = {}
 
         # The first time we shouldn't wait too long before sending heartbeat
-        self.heartbeat_delay = initial_features.get('heartbeat_delay', self.HEARTBEAT_DELAY)
+        self.heartbeat_delay = initial_features.get(
+            "heartbeat_delay", self.HEARTBEAT_DELAY
+        )
 
-        # Not used for the moment
-        self.performance_metrics_period = initial_features.get('performance_metrics_period', 0)
+        self.set_performance_metrics_settings(
+            PerformanceMetricsSettings.from_features(initial_features)
+        )
 
-        self.whitelisted_metric = initial_features.get('whitelisted_metric', True)
+        self.budget = None
+
+        self.whitelisted_metric = initial_features.get(
+            "whitelisted_metric", True
+        )
 
         self.last_heartbeat_request = 0
-        self.last_post_metrics_request = time()
 
         self.session = session
 
@@ -208,24 +247,32 @@ class Runner(object):
     def run(self):
         """ Infinite loop
         """
-        self.logger.debug("Starting runner")
+        self.logger.debug("Starting the runner (%s)",
+                          "with instrumentation enabled" if self.instrumentation.enabled
+                          else "waiting for instrumentation")
         while self.stop is False:
             self.run_once()
-        self.logger.debug("Exiting now")
+        self.logger.debug("Exiting the runner now")
 
     def run_once(self, block=True):
         """ Tries to pop a message or send an heartbeat
         """
         try:
             event = self.queue.get(timeout=self.sleep_delay, block=block)
-            self.logger.debug('Run once, event: %s after %ss sleep delay', event, self.sleep_delay)
+            self.logger.debug(
+                "Run once, event: %s after %ss sleep delay",
+                event,
+                self.sleep_delay,
+            )
             self.handle_message(event)
 
             # Exit now if should stop
             if self.stop:
                 return
         except Empty:
-            self.logger.debug('No message after %ss sleep delay', self.sleep_delay)
+            self.logger.debug(
+                "No message after %ss sleep delay", self.sleep_delay
+            )
 
         # Aggregate observations in transit in observations queue
         self.aggregate_observations()
@@ -247,9 +294,8 @@ class Runner(object):
         Process RunnerStop message or pass event to the deliverer
         """
         if event is RunnerStop:
-            self.logger.debug('RunnerStop found, logout')
+            self.logger.debug("RunnerStop found, logout")
             self.logout()
-            self.stop = True
         elif event is MetricsEvent:
             self.aggregate_observations()
         else:
@@ -265,11 +311,12 @@ class Runner(object):
         """ Do an heartbeat, publish finished metrics from MetricsStore and
         past commands results
         """
-        metrics = self.metrics_store.get_data_to_publish(datetime.utcnow(),
-                                                         force_finalize=False)
+        metrics = self.metrics_store.get_data_to_publish(
+            datetime.utcnow(), force_finalize=False
+        )
         payload = {
-            'command_results': self.next_commands_results,
-            'metrics': metrics
+            "command_results": self.next_commands_results,
+            "metrics": metrics,
         }
         res = self.session.heartbeat(payload)
         self.last_heartbeat_request = time()
@@ -277,23 +324,35 @@ class Runner(object):
         # Clean sent command results
         self.next_commands_results = {}
 
-        self.process_commands(res['commands'])
+        if res is not None:
+            self.process_commands(res["commands"])
 
     def aggregate_observations(self):
         """ Empty the observation queue and update the metric store
         with the observations in the queue.
         """
+        unregistered_metrics = set()
         try:
             while True:
-                observation = self.observation_queue.get_nowait()
-                self.metrics_store.update(*observation)
+                try:
+                    name, at, key, value = self.observation_queue.get_nowait()
+                    self.metrics_store.update(name, at, key, value)
+                except UnregisteredMetric:
+                    unregistered_metrics.add(name)
         except Empty:
             pass
+        if unregistered_metrics:
+            # metrics could be have been queued before
+            # they were disabled by a feature change
+            LOGGER.debug("Observations for unregistered metrics: %s",
+                         ", ".join(unregistered_metrics))
 
     def publish_metrics(self):
         """ Publish finished metrics from MetricsStore
         """
-        self.session.post_metrics(self.metrics_store.get_data_to_publish(datetime.utcnow()))
+        self.session.post_metrics(
+            self.metrics_store.get_data_to_publish(datetime.utcnow())
+        )
 
     def _should_do_heartbeat(self):
         """ Check if we should send an heartbeat because the delay is overdue
@@ -310,7 +369,7 @@ class Runner(object):
         self.publish_metrics()
 
         # Drain deliverer
-        self.deliverer.drain()
+        self.deliverer.drain(resiliently=False)
 
         self.session.logout()
 
@@ -324,33 +383,91 @@ class Runner(object):
     def features_get(self):
         """ Returns the current values for all features switches
         """
-        return {
-            'heartbeat_delay': self.heartbeat_delay,
-            'performance_metrics_period': self.performance_metrics_period,
-            'batch_size': self.deliverer.batch_size,
-            'max_staleness': self.deliverer.original_max_staleness,
-            'call_counts_metrics_period': self.metrics_store.get_metric_period("sqreen_call_counts"),
-            'whitelisted_metric': self.whitelisted_metric,
+        res = {
+            "heartbeat_delay": self.heartbeat_delay,
+            "batch_size": self.deliverer.batch_size,
+            "max_staleness": self.deliverer.original_max_staleness,
+            "call_counts_metrics_period": self.metrics_store.get_metric_period(
+                "sqreen_call_counts"
+            ),
+            "whitelisted_metric": self.whitelisted_metric,
         }
+        res.update(self.performance_metrics_settings.as_features())
+        return res
 
     def set_heartbeat_delay(self, heartbeat_delay):
         """ Update the heartbeat delay
         """
         self.heartbeat_delay = heartbeat_delay
 
-    def set_performance_metrics_period(self, new_performance_metrics_delay):
-        self.performance_metrics_period = new_performance_metrics_delay
+    def set_performance_cap_budget(self, budget):
+        """ Update the budget of the performance cap feature.
+        """
+        if budget is not None and budget <= 0:
+            raise ValueError("Expected a positive floating number %s" % budget)
+
+        if budget is not None and self.performance_metrics_settings.enabled():
+            LOGGER.info("Updated performance cap budget to %0.2fms", budget)
+            self.metrics_store.register_metric(
+                "request_overbudget_cb", "Sum",
+                self.performance_metrics_settings.overtime_period
+            )
+            self.budget = budget
+        else:
+            LOGGER.info("Disabled performance cap")
+            self.metrics_store.unregister_metric("request_overbudget_cb")
+            self.budget = None
+
+    def set_performance_metrics_settings(self, new_perf_met_set):
+        self.performance_metrics_settings = new_perf_met_set
+        if self.performance_metrics is not True:
+            return
+        if self.performance_metrics_settings.enabled():
+            LOGGER.info("Updated performance monitoring settings")
+            self.metrics_store.register_metric(
+                "req", "Binning", self.performance_metrics_settings.period,
+                base=self.performance_metrics_settings.base,
+                unit=self.performance_metrics_settings.unit
+            )
+            self.metrics_store.register_metric(
+                "sq", "Binning", self.performance_metrics_settings.period,
+                base=self.performance_metrics_settings.base,
+                unit=self.performance_metrics_settings.unit
+            )
+            self.metrics_store.register_metric(
+                "pct", "Binning", self.performance_metrics_settings.period,
+                base=self.performance_metrics_settings.pct_base,
+                unit=self.performance_metrics_settings.pct_unit
+            )
+            for metric_name in self.metrics_store.list_metrics():
+                # Reload performance monitoring for rules
+                if metric_name.startswith("sq."):
+                    self.metrics_store.register_metric(
+                        metric_name, "Binning",
+                        self.performance_metrics_settings.period,
+                        base=self.performance_metrics_settings.base,
+                        unit=self.performance_metrics_settings.unit
+                    )
+        else:
+            LOGGER.info("Disabled performance monitoring")
+            self.metrics_store.unregister_metric("req")
+            self.metrics_store.unregister_metric("sq")
+            self.metrics_store.unregister_metric("pct")
+            for metric_name in self.metrics_store.list_metrics():
+                if metric_name.startswith("sq."):
+                    self.metrics_store.unregister_metric(metric_name)
 
     def set_call_counts_metrics_period(self, call_counts_metrics_period):
-        self.metrics_store.register_metric("sqreen_call_counts", "Sum",
-                                           call_counts_metrics_period)
+        self.metrics_store.register_metric(
+            "sqreen_call_counts", "Sum", call_counts_metrics_period
+        )
 
     def set_whitelisted_metric(self, enabled):
         self.whitelisted_metric = enabled
 
     def set_deliverer(self, batch_size, max_staleness):
         # Drain current deliverer
-        self.deliverer.drain()
+        self.deliverer.drain(resiliently=True)
 
         # Replace current deliverer
         self.deliverer = get_deliverer(batch_size, max_staleness, self.session)
