@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2016, 2017, 2018 Sqreen. All rights reserved.
+# Copyright (c) 2016 - 2020 Sqreen. All rights reserved.
 # Please refer to our terms for more information:
 #
 #     https://www.sqreen.io/terms.html
@@ -23,14 +23,12 @@ if current_directory in sys.path:
     sys.path.remove(current_directory)
 
 try:
-    (file, pathname, description) = imp.find_module('sitecustomize', sys.path)
+    (file, pathname, description) = imp.find_module("sitecustomize", sys.path)
 except ImportError:
     # There can no other sitecustomize
     pass
 else:
-    imp.load_module('sitecustomize', file, pathname, description)
+    imp.load_module("sitecustomize", file, pathname, description)
 
 
-# Try to detect when this module is imported as sitecustomize
-if not hasattr(sys, 'argv'):
-    start()
+start()
